/*! litejs.com/MIT-LICENSE.txt */



/*
* @version    0.2.8
* @date       2014-08-24
* @stability  2 - Unstable
* @author     Lauri Rooden <lauri@rooden.ee>
* @license    MIT License
*/



!function(root) {
	var P = "prototype"
	, A = Array[P], F = Function[P], S = String[P]
	, O = Object
	, hasOwn = O[P].hasOwnProperty
	, slice = F.call.bind(A.slice)
	, constructFns = []
	, fnRe = /'(?:[^'\\]|\\.)*'|"(?:[^"\\]|\\.)*"|this|arguments|\.\w+|\w+:/g



	// Function extensions
	// -------------------


	F.construct = function(a) {
		// bind version have bad performance and memory consumption
		// return new(F.bind.apply(this, A.concat.apply([null], a)))
		var len = a.length
		return len ?
			(constructFns[len] || (constructFns[len] = Fn("t a->new t(a["+O.keys(slice(a)).join("],a[")+"])")))(this, a) :
			new this
	}

	F.partial = function() {
		var self = this
		, a = slice(arguments)
		return function() {
			return self.apply(this, a.concat.apply(a, arguments))
		}
	}

	/*
	* **argi** - index of argument, which will be split
	* **re**   - optional RegExp for matching words
	*/

	F.byWords = function(argi, re) {
		var self = this
		return function() {
			var s = this
			, r = s
			, a = arguments
			;(a[argi |= 0]||"").replace(re || /\S+/g, function(w) {
				a[argi] = w
				r = self.apply(s, a)
			})
			return r
		}
	}

	F.byKeyVal = function() {
		var self = this
		return function(o) {
			var r
			, s = this
			, a = slice(arguments)
			if (typeof o == "object") for (r in o) {
				a[0] = r
				a[1] = o[r]
				r = self.apply(s, a)
			} else r = self.apply(s, a)
			return r
		}
	}

	// Run function once and return cached value or cached instance
	F.cache = function(instance, keyFn, cache) {
		var self = this
		, c = cache || {}
		, f = function() {
			var a = arguments
			, i = !!instance || this instanceof f
			, k = keyFn ? keyFn.apply(self, a) : i + ":" + a.length + ":" + slice(a)

			return k in c ? c[k] : (c[k] = i ? self.construct(a) : self.apply(this, a))
		}

		f.origin = self
		f.cached = c
		f.extend = function() {
			return self.extend.apply(self, arguments).cache(instance, keyFn, cache)
		}
		f[P] = self[P] // prototype for better access on extending 
		return f
	}

	F.extend = function() {
		var a
		, self = this
		, i = 0

		function f() {
			return self.apply(this, arguments)
		}

		for (f[P] = O.create(self[P]); a = arguments[i++];) O.merge(f[P], a)
		f[P].constructor = f
		return f
	}

	// Time to live - Run *onTimeout* if Function not called on time
	F.ttl = function(ms, onTimeout) {
		var self = this
		, tick = setTimeout(function(){ms=0;onTimeout&&onTimeout()}, ms)

		return function() {
			clearTimeout(tick)
			ms && self.apply(null, arguments)
		}
	}

	// Run Function one time after last call
	F.once = function(ms) {
		var tick, args
		, self = this
		return function() {
			clearTimeout(tick)
			args = arguments
			tick = setTimeout(function(){self.apply(null, args)}, ms)
		}
	}

	// Maximum call rate for Function
	F.rate = function(ms, last_call) {
		var tick, args
		, self = this, next = 0
		return function() {
			var now = +new Date()
			clearTimeout(tick)
			if (now > next) {
				next = now + ms
				self.apply(null, arguments)
			} else if (last_call) {
				args = arguments
				tick = setTimeout(function(){self.apply(null, args)}, next-now)
			}
		}
	}


	// Non-standard
	O.each = function(obj, fn, scope, key) {
		if (obj) for (key in obj) hasOwn.call(obj, key) && fn.call(scope, obj[key], key, obj)
	}

	// Object.assign ( target, source ) in ECMAScript 6

	O.merge = function(target, source) {
		for (var key, i = 1; source = arguments[i++];)
			for (key in source) if (hasOwn.call(source, key)) target[key] = source[key]
		return target
	}

	// Note: use for Object literals only,
	// as it returns false for custom objects,
	// like new Date or new YourObject.

	function isObject(obj) {
		return obj && obj.constructor === O
	}

	O.clone = function(source, temp, key) {
		if (isObject(source)) {
			temp = {}
			for (key in source) if (hasOwn.call(source, key))
				temp[key] = O.clone(source[key])
			source = temp
		}
		return source
	}

	O.deepMerge = O.deepCopy = function(target, source, path, changed, key, val) {
		path = path || ""
		changed = changed || []

		for (key in source) if (hasOwn.call(source, key) && target[key] !== source[key]) {
			val = source[key]
			changed.push(path + key)
			if (val === null) delete target[key]
			else if (isObject(val)) {
				if (!isObject(target[key])) target[key] = {}
				O.deepMerge(target[key], val, path + key + ".", changed)
			}
			else target[key] = val
		}
		return changed
	}

	O.zip = function(keys, vals) {
		return keys.fold(function(_, key, i) {
			_[key] = vals[i]
			return _
		}, {})
	}

	/*
	Array.flatten = function(arr) {
	for(var i=arr.length;i--;)
	0 in arr[i] && A.splice.apply(arr, [i, 1].concat(Array.flatten(arr[i])))
	return arr
	}
	flat([1,2,[3,4,[5,6]],7])
	*/
	// Non-standard
	// // IE < 9 bug: [1,2].splice(0).join("") == "" but should be "12"
	A.remove = function() {
		var arr = this
		, len = arr.length
		, o = slice(arguments)
		, lastId = -1

		for (;len--;) if (~o.indexOf(arr[len])) arr.splice(lastId = len, 1)
		return lastId
	}

	A.each = A.forEach
	A.fold = A.reduce
	A.foldr = A.reduceRight
	// uniq
	// last item preserved
	A.uniq = A.filter.partial(function(s,i,a){return i == a.lastIndexOf(s)})

	A.pushUniq = function(item) {
		return this.indexOf(item) == -1 ? this.push(item) : false
	}

	!function(n) {
		F[n] = S[n] = function() {
			var a = arguments, l = a[0]
			a[0] = this.fn()
			return A[n].apply(l, a)
		}
	}.byWords()("every filter each map fold foldr some")


	F.fn = function() {
		return this
	}

	S.fn = function(scope) {
		return Fn(this, scope)
	}

	// THANKS: Oliver Steele http://www.osteele.com/sources/javascript/functional/
	function Fn(expr, scope) {
		for (var args = "_", body = expr, arr = expr.split("->"); arr.length > 1; ) {
			body = arr.pop()
			args = arr.pop().match(/\w+/g) || ""
			if (arr.length) arr.push("(function("+args+"){return("+body+")})")
		}
		// `replace` removes symbols that follow '.',
		//  precede ':', are 'this' or 'arguments'; and also the insides of
		//  strings (by a crude test).  `match` extracts the remaining
		//  symbols.
		return new Function(args, (scope && (expr = expr.replace(fnRe, "").match(/\b[a-z]\w*|\b_\w+/g)) ?
			"var " + expr.uniq().join("='',") + "='';with(" + scope + "||{})" : "") + "return(" + body + ")")
	}

	root.Fn = Fn.cache()

}(this)







/**
 * @version    0.3.2
 * @date       2014-09-12
 * @stability  1 - Experimental
 * @author     Lauri Rooden <lauri@rooden.ee>
 * @license    MIT License
 */



// TODO: find ways for automated testing
// http://www.browserscope.org/user/tests/howto


!function(window, doc, protoStr) {
	var elCache = {}
	, fnCache = {}
	, proto = (window.HTMLElement || window.Element || El)[protoStr]
	, elRe = /([.#:[])([-\w]+)(?:=((["'\/])(?:\\.|.)*?\4|[-\w]+)])?]?/g
	, tplRe = /^([ \t]*)(\:?)((?:(["'\/])(?:\\.|.)*?\4|[-\w\:.#\[\]=])+)[ \t]*(.*)$/gm
	, renderRe = /[;\s]*(\w+)(?:\s*\:((?:(["'\/])(?:\\.|.)*?\3|[-,\s\w])*))?/g
	, bindings = El.bindings = {
		"txt": function(node, data, text) {
			node.txt(text.format(data))
		},
		"class": function(node, data, name, fn) {
			node.toggleClass(name, fn.fn("_")(data))
		},
		"html": function(node, data, html) {
			node.innerHTML = html.format(data)
		},
		/**
		"with": function(node, data, scope) {
			render.call(node, scope, true)
		},
		"if": function(node, data, fn) {
			var childs = getChilds(node)
			node.empty().append( fn.fn("_")(data) && childs )
		},
		//*/
		"each": function(node, data, arr) {
			var childs = getChilds(node)
			if (arr) node.empty().append(arr.map(function(obj) {
				return childs.map(function(el) {
					return render.call(el.cloneNode(true), obj)
				})
			}))
			return node
		}
	}


	/*
	 * Examples:
	 *   - El("input#123.nice[type=checkbox]:checked:disabled[data-lang=en]")
	 */

	function El(name, args) {
		var el
		, pre = {}
		name = name.replace(elRe, function(_, op, key, val, quotation) {
			pre[
				op == "." ? (op = "class", (pre[op] && (key = pre[op]+" "+key)), op) :
				op == "#" ? "id" :
				key
			] = (quotation ? val.slice(1, -1): val) || key
			return ""
		}) || "div"

		// NOTE: IE-s cloneNode consolidates the two text nodes together as one
		// http://brooknovak.wordpress.com/2009/08/23/ies-clonenode-doesnt-actually-clone/
		el = (elCache[name] || (elCache[name] = doc.createElement(name))).cloneNode(true).set(pre)

		return fnCache[name] && fnCache[name].call(el, args) || el.set(args)
	}


	// TODO: Extend El api
	// add El.siblings( [selector ] )
	// add El.children( [selector ] )
	// add El.invoke
	// https://github.com/WebReflection/dom4#dom4

	//
	// How elements.js extends the DOM
	// -------------------------------
	//
	// All DOM extensions on the element are available by default.
	//
	// In browsers that does not support adding methods to prototype of native objects
	// such as HTMLElement or Element, document.createElement will be overrided
	// to extend created elements. El.get() and element.find will extend
	// returned elements.

	// ### element.append( content, [ before ] ) -> element
	//
	// - **content** `element || Array || String || Number`
	// - **before** `optional`
	//     - true - Insert content to the beginning of element
	//     - element - Insert content before specified element
	//     - Number - Insert content before nth child

	function append(child, before) {
	       var el = this
	       if (child) {
			if (typeof child == "string" || typeof child == "number") child = El.text(child)
			else if ( !("nodeType" in child) && "length" in child ) {
				// document.createDocumentFragment is unsupported in IE5.5
				// fragment = "createDocumentFragment" in doc ? doc.createDocumentFragment() : El("div")
				for (
					var len = child.length
					, i = 0
					, fragment = doc.createDocumentFragment();
					i < len; ) append.call(fragment, child[i++]);
				child = fragment
			}

			if (child.nodeType) el.insertBefore(child,
				(before === true ? el.firstChild :
				typeof before == "number" ? el.childNodes[
					before < 0 ? el.childNodes.length - before - 2 : before
				] : before) || null
			)
			if (child.appendHook) child.appendHook()
			//"childHook" in el && el.childHook()
		}
		return el
	}
	proto.append = append

	proto.after = function(silbing, before) {
		// call append so it works with DocumentFragment
		append.call(silbing.parentNode, this, before ? silbing : silbing.nextSibling)
		return this
	}

	proto.to = function(parent, before) {
		append.call(parent, this, before)
		return this
	}

	function hasClass(name) {
		// http://jsperf.com/regexp-indexof-perf/32
		// return (" "+this.className+" ").indexOf(" "+name+" ") > -1
		return RegExp("\\b" + name + "\\b").test(this.className)
	}
	proto.hasClass = hasClass

	function addClass(name) {
		var el = this
		el.className += !el.className ? name : hasClass.call(el, name) ? "" : " " + name
		return el
	}
	proto.addClass = addClass

	function rmClass(name) {
		var el = this
		el.className = (" "+el.className+" ").replace(" "+name+" "," ").trim()
		return el
	}
	proto.rmClass = rmClass

	proto.toggleClass = function(name, force) {
		if (arguments.length == 1) force = !hasClass.call(this, name)
		return ( force ? addClass : rmClass ).call(this, name), force
	}

	proto.empty = function() {
		for (var node, el = this; node = el.firstChild; ) kill.call(node)
		return el
	}

	function kill() {
		var el = this
		if (el.parentNode) el.parentNode.removeChild(el)
		if (Event.removeAll) Event.removeAll(el)
		if (el.killHook) el.killHook()
		if (el.empty) el.empty()
		return el
	}
	proto.kill = kill

	proto.on = function(ev, fn) {
		Event.add(this, ev, fn)
		return this
	}

	proto.non = function(ev, fn) {
		Event.remove(this, ev, fn)
		return this
	}

	proto.set = function(args) {
		var val
		, el = this
		, key = typeof args

		if (!args) return el
		if (key == "string" || key == "number" || args.nodeType || "length" in args) append.call(el, args)
		else for (key in args) {
			val = args[key]
			// El uses class
			if (key == "class") addClass.call(el, val)
			else if (!val) el.removeAttribute(key)
			else if (typeof val == "string") {
				// Note: IE5-7 doesn't set styles and removes events when you try to set them.
				//
				// in IE6, a label with a for attribute linked to a select list
				// will cause a re-selection of the first option instead of just giving focus.
				// http://webbugtrack.blogspot.com/2007/09/bug-116-for-attribute-woes-in-ie6.html
				el.setAttribute(key, val)

				// there are bug in IE<9 where changed 'name' param not accepted on form submit
				// The JScript engine used in IE doesn't recognize vertical tabulation character
				// oldIE = "\v" == "v"
				//
				// IE8 and below also support document.createElement('<P>')
				//
				// http://www.matts411.com/post/setting_the_name_attribute_in_ie_dom/
				// http://msdn.microsoft.com/en-us/library/ms536614(VS.85).aspx

				if ((key == "id" || key == "name") && "\v" == "v") {
					el.mergeAttributes(doc.createElement('<INPUT '+key+'="' + val + '"/>'), false)
				}
			} else el[key] = val
		}
		return el
	}

	// Save node initial content for later use
	function getChilds(node) {
		var child
		, childs = node._childs
		if (!childs) {
			node._childs = childs = []
			for (; child = node.firstChild;) {
				childs.push(child);
				node.removeChild(child)
			}
		}
		return childs
	}

	function render(data, skipSelf) {
		var bind, fn, lang
		, node = this

		if (bind = !skipSelf && node.getAttribute("data-call")) {
			console.log("node", node, bind)
			fnCache[bind].call(node)
		}
		if (bind = !skipSelf && node.getAttribute("data-bind")) {
			lang = node.getAttribute("lang") || lang
			// HACK: allow .el ={name} short syntax
			if (bind.charAt(0) == "{") bind='txt:"' + bind.replace(/"/g, '\\"') + '"'
			// i18n(bind, lang).format(data)
			// document.documentElement.lang
			// document.getElementsByTagName('html')[0].getAttribute('lang')

			fn = "n d p r->d&&(" + bind.replace(renderRe, "(p['$1']?(r=p['$1'](n,d,$2)||r):(n['$1']=$2.format(d))),") + "r)"

			if (fn.fn()(node, data, bindings)) return node
		}

		for (node = node.firstChild; node; node = node.nextSibling) {
			if (node.nodeType == 1) render.call(node, data)
		}
		return this
	}

	proto.render = render

	// In Safari 2.x, innerText functions properly only
	// if an element is neither hidden (via style.display == "none")
	// nor orphaned from the document.
	// Otherwise, innerText results in an empty string.
	//
	// textContent is suported from IE9
	//
	// Opera 9-10 have Node.text, Node.textContent

	proto.txt = function(newText) {
		var el = this
		, attr = "textContent" in el ? "textContent" : "innerText"
		return arguments.length ? (el[attr] = newText) : el[attr]
	}

	// NOTE: fast selectors for IE
	// http://ajaxian.com/archives/creating-a-queryselector-for-ie-that-runs-at-native-speed

	function findEl(node, sel, first) {
		var el
		, i = 0
		, out = []
		, rules = ["_"]
		, tag = sel.replace(elRe, function(_, o, s, v) {
			rules.push(
				o == "." ? "(' '+_.className+' ').indexOf(' "+s+" ')>-1" :
				o == "#" ? "_.id=='"+s+"'" :
				"_.getAttribute('"+s+"')"+(v?"=='"+v+"'":"")
			)
			return ""
		}) || "*"
		, els = node.getElementsByTagName(tag)
		, fn = Fn(rules.join("&&"))

		for (; el = els[i++]; ) if (fn(el)) {
			if (first) return el
			out.push(el)
		}
		return first ? null : out
	}

	// Note: IE8 don't support :disabled
	// TODO: test with IE, should it be proto.querySelector or body.querySelector
	proto.find = "\v" !== "v" && proto.querySelector || function(sel) {
		return findEl(this, sel, true)
	}

	proto.findAll = proto.querySelectorAll ?
		function(sel) {
			return new ElAll(this.querySelectorAll(sel))
		} :
		function(sel) {
			return new ElAll(findEl(this, sel))
		}

	function ElAll(nodes) {
		this._nodes = nodes
	}

	ElAll.prototype = Object.keys(proto).reduce(function(memo, key) {
		memo[key] = function() {
			var elAll = this
			, nodes = elAll._nodes
			, i = 0
			, len = nodes.length
			for (; i < len; ) {
				proto[key].apply(nodes[i++], arguments)
			}
			return elAll
		}
		return memo
	}, {})


	function extend(node, key) {
		if (node) for (key in proto) node[key] = proto[key]
		return node
	}


	// IE 6-7
	if (proto === El[protoStr]) {
		var create = doc.createElement
		doc.createElement = function(name) {return extend(create(name))}

		// NOTE: document.body will not get extended with later added extensions
		extend(doc.body)

		// Remove background image flickers on hover in IE6
		//
		// You could also use CSS
		// html { filter: expression(document.execCommand("BackgroundImageCache", false, true)); }
		/*@cc_on try{document.execCommand('BackgroundImageCache',false,true)}catch(e){}@*/
	}
	window.El = El

	El[protoStr] = proto

	El.get = function(id) {
		if (typeof id == "string") id = doc.getElementById(id)
		return id && id.to ? id : extend(id)
	}

	function elCacheFn(name, el, custom) {
		elCache[name] = typeof el == "string" ? El(el) : el
		if (custom) {
			if (custom != render) el.setAttribute("data-call", name)
			fnCache[name] = custom
		}
	}

	elCacheFn._el = elCache
	elCacheFn._fn = fnCache
	El.cache = elCacheFn

	El.text = function(str) {
		return doc.createTextNode(str)
	}


	function tpl(str) {
		var root = document.createDocumentFragment()
		, parent = root
		, stack = [-1]

		function work(all, indent, plugin, name, q, text) {
			for (var i = indent.length; i <= stack[0]; ) {
				stack.shift()
				parent = (parent.plugin) ? parent.plugin.done() : parent.parentNode
			}

			if (plugin) {
				if (tpl.plugins[name]) {
					parent = (new tpl.plugins[name](parent, text)).el
					stack.unshift(i)
				} else {
					parent.append(El.text( name == "text" ? text : all ))
				}
			} else {
				if (name) {
					parent = El(name).to(parent)
					stack.unshift(i)
				}
				if (text) {
					if (text.charAt(0) == ">") {
						(indent +" "+ text.slice(1)).replace(tplRe, work)
					} else if (text.charAt(0) == "=") {
						parent.set({"data-bind": text.slice(1)})
					} else {
						parent.append(text.replace(/\\([=>:])/g, "$1"))
					}
				}
			}
		}
		str.replace(tplRe, work)
		stack = root.childNodes
		if (stack.length == 1) return stack[0]

		for (var arr = [], i = stack.length; i--;) arr[i] = stack[i]
		return arr
	}

	function template(parent, name) {
		var t = this
		t.name = name
		t.parent = parent
		t.el = El("div")
		t.el.plugin = t
		return t
	}

	template.prototype.done = function() {
		var t = this

		El.cache(t.name, t.el.removeChild(t.el.firstChild), render)

		t.el.plugin = null

		return t.parent
	}

	tpl.plugins = {
		/*
		 * - Declaration
		 * mixin list
		 *
		 * mixin link(href, name)
		 *   a(class!=attributes.class, href=href)= name
		 *
		 * +link('/foo', 'foo')(class="btn")
		 *
		 * :include
		 * :doctype
		 * - http://stackoverflow.com/questions/8227612/how-to-create-document-objects-with-javascript
		 */
		"template": template
	}

	El.tpl = tpl
	El.include = function(id, data, parent) {
		var src = El.get(id)
		new template(null, id).el.append( El.tpl(src.innerHTML) ).plugin.done()
		src.kill()
	}
}(window, document, "prototype")






/*
* @version    0.2.2
* @date       2014-05-24
* @stability  1 - Experimental
* @author     Lauri Rooden <lauri@rooden.ee>
* @license    MIT License
*/



/*
 * In IE6, most of the choices in the Language Preference list specified a locale-neutral two letter code.
 * For instance fr was sent for French (France), and ja sent for Japanese.
 * Longer codes were only used when a language is commonly spoken in another country or locale-- for instance fr-CA was available for French-speaking Canadians.
 *
 * For Internet Explorer 7, a change was made such that Internet Explorer will send the full language/locale pair for each locale.
 * IE7 will send fr-FR for French (France) and de-DE for German (Germany).
 * This change enables web servers to more easily target content for a specific language and locale.
 * If a given server is only interested in the user’s language and not the locale, it can ignore the locale portion by simply truncating the code at the first dash.
 *
 * Anyway, it's always more user-friendly to use Accept-Language than IP-address-based Geo-Location.
 *
 * Accept-Language: eo;q=0
 * the q stands for "quality" and denotes a preference where 1 is highest and 0 means "not acceptable".
 * means "I can't read or understand Esperanto and no, please do not provide any content in that language."
 */


!function(root) {
	var current;

	function i18n(text, lang) {
		return (i18n[lang ? getLang(lang) : current][text] || text)
	}

	function getLang(lang) {
		if (!lang) return current
		lang = (lang||"").toLowerCase()
		return i18n[lang] ? lang : (lang = lang.split("-")[0]), i18n[lang] ? lang : current
	}

	function setLang(lang) {
		lang = i18n.getLang(lang)
		if (current != (current = lang)) {
			i18n[lang] = i18n[lang] || {}
		}
		return lang
	}

	i18n.getLang = getLang
	i18n.setLang = setLang

//i18n.main = i18n.current = "en"

i18n.en = {
	date: "%a, %d %b %Y %H:%M:%S %z",
	name: "Name {date|lang}"
}

i18n.et = {
	date: "%Y %H:%M:%S %z",
	name: "Nimi {date|lang:'et'}"
}

//i18n.setLang(navigator.language || navigator.userLanguage)

Date.prototype.lang = function(lang) {
	return this.format( i18n("date", lang) )
}

String.prototype.lang = function(lang) {
	return i18n(this, lang)
}
	root.i18n = i18n
}(this)





/*
* @version    0.1.4
* @date       2014-03-10
* @stability  2 - Unstable
* @author     Lauri Rooden <lauri@rooden.ee>
* @license    MIT License
*/



!function(root) {
	// The addEventListener is supported in Internet Explorer from version 9.
	// https://developer.mozilla.org/en-US/docs/Web/Reference/Events/wheel
	// - IE8 always prevents the default of the mousewheel event.

	var doc = document
	, Event = root.Event || (root.Event={})
	, wheelDiff = 120
	, addEv = "addEventListener"
	, remEv = "removeEventListener"
	, prefix = root[addEv] ? "" : (addEv = "attachEvent", remEv = "detachEvent", "on")
	, WHEEL_EVENT = 
		"onwheel" in doc      ? "wheel" :      // Modern browsers support "wheel"
		"onmousewheel" in doc ? "mousewheel" : // Webkit and IE support at least "mousewheel"
		"DOMMouseScroll"                       // let's assume that remaining browsers are older Firefox

	, Emitter = Event.Emitter = {
		on: function(ev, fn, scope) {
			var t = this
			, e = t._e || (t._e = {})
			;(e[ev] || (e[ev] = [])).push([fn, scope])
			return t
		},
		non: function(ev, fn) {
			var t = this
			if (ev) {
				if (t._e && t._e[ev]) {
					if (fn) for (var a = t._e[ev], l = a.length; l--;) if (a[l][0] == fn) a.splice(l, 1)
					else delete t._e[ev]
				}
			} else delete t._e
			return t
		},
		once: function(ev, fn, scope) {
			var t = this
			return t.on(ev, fn, scope).on(ev, t.non.bind(t, ev, fn))
		},
		emit: function(ev) {
			var t = this
			if (t._e && t._e[ev]) {
				for (var i=0, e=t._e[ev], a=e.slice.call(arguments, 1); ev=e[i++];) ev[0].apply(ev[1]||t, a)
			}
			return t
		}
	}

	// Alias
	Emitter.off = Emitter.non


	Event.add = function(el, ev, _fn) {
		var fn = ev == "wheel" ? 
			function(e) {
				if (!e) e = root.event
				var delta = (e.wheelDelta || -e.detail || -e.deltaY)/wheelDiff
				if (delta != 0) {
					if (delta < 1 && delta > -1) {
						var diff = (delta < 0 ? -1 : 1)/delta
						delta *= diff
						wheelDiff /= diff
					}
					//TODO: fix event
					// e.deltaY = 
					// e.deltaX = - 1/40 * e.wheelDeltaX|0
					// e.target = e.target || e.srcElement
					_fn.call(el, e, delta)
				}
			} : 
			prefix ? function(){
				_fn.call(el, root.event)
			} : _fn

		if (fn != _fn) fn.origin = _fn
		Emitter.on.call(el, ev, fn, el)


		el[addEv](prefix + (ev == "wheel" ? WHEEL_EVENT : ev), fn, false)
		return Event
	}

	Event.remove = function(el, ev, fn) {
		if (fn && el._e && el._e[ev]) {
			for (var _fn, arr = el._e[ev], i = 0; _fn=arr[i];i++) { 
				_fn = _fn[0]
				if (_fn == fn || _fn.origin == fn) {
					arr.splice(i, 1)
					fn = _fn
				}
			}
		}
		el[remEv](prefix + (ev == "wheel" ? WHEEL_EVENT : ev), fn, false)
		return Event
	}

	Event.stop = function(e) {
		e.stopPropagation && e.stopPropagation()
		e.preventDefault && e.preventDefault()
		e.cancelBubble = e.cancel = true
		return e.returnValue = false
	}

	Event.removeAll = function(el, ev, key, arr, i) {
		if (el._e) for (key in el._e) {
			if (!ev || key == ev) for (arr = el._e[key], i = arr.length; i--;) {
				Event.remove(el, key, arr[0])
			}
		}
	}

}(this)




/*! litejs.com/MIT-LICENSE.txt */



/**
 * @version    0.1.1
 * @date       2014-09-02
 * @stability  2 - Unstable
 * @author     Lauri Rooden <lauri@rooden.ee>
 * @license    MIT License
 */



// https://github.com/browserstate/history.js/wiki/The-State-of-the-HTML5-History-API

!function(win, doc, his) {
	var cb, base, last_route, iframe, tick, last
	, loc = location
	, clean_route = /^[#\/\!]+|[\s\/]+$/g
	/*
	* The JScript engine used in IE doesn't recognize vertical tabulation character
	* http://webreflection.blogspot.com/2009/01/32-bytes-to-know-if-your-browser-is-ie.html
	* oldIE = "\v" == "v"
	*
	* The documentMode is an IE only property, supported in IE8+.
	*
	* Starting in Internet Explorer 9 standards mode, Internet Explorer 10 standards mode, 
	* and win8_appname_long apps, you cannot identify the browser as Internet Explorer 
	* by testing for the equivalence of the vertical tab (\v) and the "v". 
	* In earlier versions, the expression "\v" === "v" returns true. 
	* In Internet Explorer 9 standards mode, Internet Explorer 10 standards mode, 
	* and win8_appname_long apps, the expression returns false.
	*/
	, ie6_7 = !+"\v1" && (doc.documentMode||1) < 8

	function getUrl(_loc) {
		var url
		/*** PUSH
		if (base) {
			url = loc.pathname.slice(base.length)
		} else {
		//*/
			// bug in Firefox where location.hash is decoded
			// bug in Safari where location.pathname is decoded

			// var hash = loc.href.split('#')[1] || '';
			// https://bugs.webkit.org/show_bug.cgi?id=30225
			// https://github.com/documentcloud/backbone/pull/967
			url = (_loc || loc).href.split("#")[1] || ""
		/*** PUSH
		}
		//*/
		// Fix decode
		// http://unixpapa.com/js/querystring.html
		url = decodeURIComponent(url.replace(/\+/g, " "))
		return url.replace(clean_route, "")
	}

	function setUrl(url, replace) {
		url = encodeURIComponent(url).replace(/%20/g, "+").replace(/%2F/gi, "/")
		/*** PUSH
		if (base) {
			his[replace ? "replaceState" : "pushState"](null, null, base + url)
		} else {
		//*/
			loc[replace ? "replace" : "assign"]("#" + url)
			// Opening and closing the iframe tricks IE7 and earlier 
			// to push a history entry on hash-tag change.
			if (iframe && getUrl() !== getUrl(iframe.location) ) {
				iframe.location[replace ? "replace" : iframe.document.open().close(), "assign"]("#" + url)
			}
		/*** PUSH
		}
		//*/
		checkUrl()
	}

	function checkUrl() {
		if (last_route != (last_route = getUrl()) && cb) cb(last_route)
	}

	his.getUrl = getUrl
	his.setUrl = setUrl

	his.start = function(_cb, _base) {
		if (!cb) {
			/*** PUSH
			if (_base && his.pushState) {
				base = _base
				// Chrome and Safari emit a popstate event on page load, Firefox doesn't.
				// Firing popstate after onload is as designed.
				//
				// See the discussion on https://bugs.webkit.org/show_bug.cgi?id=41372, 
				// https://code.google.com/p/chromium/issues/detail?id=63040
				// and the change to the HTML5 spec that was made: 
				// http://html5.org/tools/web-apps-tracker?from=5345&to=5346. 
				win.onpopstate = checkUrl
			} else
			//*/
				if ("onhashchange" in win && !ie6_7) {
				// There are onhashchange in IE7 but its not get emitted
				//
				// Basic support:
				// Chrome 5.0, Firefox (Gecko) 3.6 (1.9.2), IE 8.0, Opera 10.6, Safari 5.0
				win.onhashchange = checkUrl
			} else {
				if (ie6_7) {
					// IE<9 encounters the Mixed Content warning when the URI javascript: is used.
					// IE5/6 additionally encounters the Mixed Content warning when the URI about:blank is used.
					iframe = doc.body.appendChild(doc.createElement('<iframe class="hide" src="//:" tabindex="-1">')).contentWindow
				}
				last = getUrl()
				tick = setInterval(function(){
					var cur = getUrl()
					if (iframe && last === cur) cur = getUrl(iframe.location)
					if (last !== cur) {
						last = cur
						iframe ? setUrl(cur) : checkUrl()
					}
				}, 50)
			}
		}
		cb = _cb
		checkUrl()
	}
}(this, document, history)







/**
 * @version    0.3.1
 * @date       2014-09-12
 * @stability  1 - Experimental
 * @author     Lauri Rooden <lauri@rooden.ee>
 * @license    MIT License
 */





!function(exports) {
	var fn, re
	, fnStr = ""
	, reStr = ""
	, views = {}
	, groupsCount = 1
	, escapeRe = /[.*+?^=!:${}()|\[\]\/\\]/g
	, parseRe = /\{([\w%.]+?)\}|.[^{\\]*?/g

	function escapeRegExp(string) {
		return string.replace(escapeRe, "\\$&")
	}

	function View(route, el, parent, selector) {
		var view = this
		if (views[route]) return views[route]
		if (!(view instanceof View)) return new View(route, el, parent, selector)
		views[view.route = route] = view
		view.el = el
		view.parent = typeof parent == "string" ? View(parent) : parent
		view.selector = selector

		if (route.charAt(0) != "#") {
			var startLen = groupsCount++
			, a = ["a._r='" + route + "'"]
			, _re = route.replace(/:(\w+)/g, "{$1}").replace(parseRe, function(_, key) {
				return key ?
					a.push("a['" + key + "']=a[" + (groupsCount++) + "]") && "([^/]+?)" :
					escapeRegExp(_)
			})

			fnStr += (fnStr ? "||" : "") + "a[" + startLen + "]&&(" + a + ")"
			reStr += (reStr ? "|" : "") + "(" + _re + ")"

			fn = new Function("a", "return " + fnStr + ",a")
			re = new RegExp("^\\/?(?:" + reStr + ")[\\/\\s]*$")
		}

		if (View.active == route) view.show()
	}

	View.prototype = {
		getEl: function() {
			var view = this
			, type = typeof view.el
			if (type == "function") view.el = view.el()
			else if (type == "string") view.el = El(view.el)
			return view.el
		},
		getContentEl: function() {
			var view = this
			return view.selector && view.getEl().find(view.selector) || view.getEl()
		},
		load: function(opts) {
			var view = this
			xhr.load((View.base || "") + (view.file || view.route + ".js"), function() {
				if (view.el) view.show(opts)
				else throw new Error("No View " + view.route)
			})
		},
		show: function(opts) {
			var view = this
			View.active = view.route
			if (view.el) {
				if (view.active) view.close()
				view.ping(opts)
				if (View.active == view.route) view.emit("show", opts)
			} else {
				view.load(opts)
			}
		},
		close: function(opts) {
			var view = this
			, parent = view.parent

			view.active = false

			if (parent) {
				parent.getContentEl().removeChild(view.getEl())
				parent.child = null
			}
			view.emit("close", opts)
			if (view.child) view.child.close()
		},
		ping: function(opts) {
			var view = this
			, parent = view.parent

			view.active = true

			if (parent) {
				if (parent.child != view) {
					if (parent.child) parent.child.close()
					parent.child = view
					parent.getContentEl().appendChild(view.getEl())
				}
				parent.ping(opts)
			}
			view.emit("ping", opts)
		}
	}

	Object.merge(View.prototype, Event.Emitter)

	// TODO: "default" is a reserved word, pick new one.
	// View.index
	// View.init
	// View.home
	// View.main
	// View.root
	// View.start
	View.show = function(route) {
		var match = re.exec(route)
		View(
			match && (match = fn(match))._r ||
			route && View["404"] ||
			View.main || View["default"]
		).show(match)
	}

	exports.View = View
}(this)

